@@ -9,7 +9,6 @@
 #include <sys/signalfd.h>
 #include <sys/ptrace.h>
 #include <sys/wait.h>
-#include <sys/socket.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <linux/if.h>
@@ -746,7 +745,7 @@ static int check_aio_remap(void)
 	if (!len)
 		return -1;
 
-	naddr = mmap(NULL, len, PROT_READ | PROT_WRITE, MAP_ANON | MAP_PRIVATE, 0, 0);
+	naddr = mmap(NULL, len, PROT_READ | PROT_WRITE, MAP_ANONYMOUS | MAP_PRIVATE, 0, 0);
 	if (naddr == MAP_FAILED) {
 		pr_perror("Can't find place for new AIO ring");
 		return -1;
@@ -1086,6 +1085,13 @@ static int check_kcmp_epoll(void)
 	return 0;
 }
 
+static int check_net_diag_raw(void)
+{
+	check_sock_diag();
+	return (socket_test_collect_bit(AF_INET, IPPROTO_RAW) &&
+		socket_test_collect_bit(AF_INET6, IPPROTO_RAW)) ? 0 : -1;
+}
+
 static int (*chk_feature)(void);
 
 /*
@@ -1194,6 +1200,7 @@ int cr_check(void)
 		ret |= check_uffd_noncoop();
 		ret |= check_sk_netns();
 		ret |= check_kcmp_epoll();
+		ret |= check_net_diag_raw();
 	}
 
 	/*
@@ -1292,6 +1299,7 @@ static struct feature_list feature_list[] = {
 	{ "can_map_vdso", check_can_map_vdso},
 	{ "sk_ns", check_sk_netns },
 	{ "sk_unix_file", check_sk_unix_file },
+	{ "net_diag_raw", check_net_diag_raw },
 	{ "nsid", check_nsid },
 	{ "link_nsid", check_link_nsid},
 	{ "kcmp_epoll", check_kcmp_epoll},
