@@ -43,7 +43,10 @@ int main(int argc, char ** argv)
 	mkdir(src, 755);
 	mkdir(dst, 755);
 
-	unshare(CLONE_NEWNS);
+	if (unshare(CLONE_NEWNS)) {
+		pr_perror("unshare");
+		return 1;
+	}
 
 	if (mount(src, dst, NULL, MS_BIND, NULL)) {
 		pr_perror("bind");
